/*

JSON UDFs - convert XML to JSON and vice versa

This code is intended to run as Java external UDFs (using DB2, etc.)


To Compile (assuming JSON_UDFs.java is in folder /myfolder):
 
1. Start QShell  (QSH)
2. Use the javac command as follows (the Java file name is case sensitive!):
   javac -d /qibm/userdata/os400/sqllib/function -cp /tmp/java-json.jar /tmp/JSON_UDFs.java



SQL Function Registration JSON2XML UDF   (replace QGPL as appropriate)
-----------------------------------------------------------------------
CREATE OR REPLACE FUNCTION QGPL.JSON2XML(JSONData CLOB)
RETURNS CLOB
SPECIFIC QGPL.JSON2XML_CLOB
EXTERNAL NAME 'JSON_UDFs.convertJSON2XML'
LANGUAGE Java
PARAMETER STYLE Java
FENCED
NO SQL
RETURNS NULL ON NULL INPUT
SCRATCHPAD
DETERMINISTIC
;

Sample usage:
--------------------------------
SELECT QGPL.JSON2XML(CLOB(
'{
    "id": 1,
    "name": "A green door",
    "price": 12.50,
    "tags": ["home", "green"]
}'))
FROM SYSIBM.SYSDUMMY1;


SQL Function Registration XML2JSON UDF   (replace QGPL as appropriate)
-----------------------------------------------------------------------
CREATE OR REPLACE FUNCTION QGPL.XML2JSON(XMLData CLOB)
RETURNS CLOB
SPECIFIC QGPL.XML2JSON_CLOB
EXTERNAL NAME 'JSON_UDFs.convertXML2JSON'
LANGUAGE Java
PARAMETER STYLE Java
FENCED
NO SQL
RETURNS NULL ON NULL INPUT
SCRATCHPAD
DETERMINISTIC


SELECT QGPL.XML2JSON(CLOB(
'<tags>home</tags><tags>green</tags><id>1</id><price>12.5</price><name>A green door</name>'))
FROM SYSIBM.SYSDUMMY1;

*/
import org.json.*;
import java.sql.*;
import java.io.IOException;

public class JSON_UDFs {

  public JSON_UDFs() {

  }

  public static java.sql.Clob convertJSON2XML(java.sql.Clob jsonData) throws JSONException, SQLException, IOException {
        if (jsonData == null) {
            return null;
        }
        long size = jsonData.length();
        char[] cbuff = new char[(int)size];
        if (jsonData.getCharacterStream().read(cbuff) == -1)
            return null;
        JSONObject json = new JSONObject(new String(cbuff));
        String data = XML.toString(json);
        jsonData.setString(1, data);
        jsonData.truncate(data.length());
        return jsonData;
  }

  public static java.sql.Clob convertXML2JSON(java.sql.Clob xml) throws JSONException, SQLException, IOException {
        if (xml == null) {
            return null;
        }
        long size = xml.length();
        char[] cbuff = new char[(int)size];
        if (xml.getCharacterStream().read(cbuff) == -1)
            return null;
        JSONObject json = XML.toJSONObject(new String(cbuff));
        String data = json.toString();
        xml.setString(1, data);
        xml.truncate(data.length());
        return xml;
  }
}